<?php

use Nette\Forms\Form;
use Instante\Bootstrap3Renderer\BootstrapRenderer;

function pack_scores($scores) {
    $blob = '';
    foreach ($scores as $score)
        $blob .= pack('C', $score);
    return $blob;
}

function unpack_scores($blob) {
    return array_map(function($c) { return unpack('C', $c)[1]; }, str_split($blob));
}

function match_arrows($row) {
    return sprintf('%d &times; %d', $row->turns, $row->arrows);
}

function match_score($row) {
    return array_sum(unpack_scores($row->scores));
}

function match_avg_score($row) {
    return match_score($row) / ($row->turns * $row->arrows);
}

function find_match($id) {
    global $app, $db, $user;
    $match = $db->table('match')->get($id);

    if (!$match)
        $app->halt(404, 'Match does not exist');

    if ($match->user_id != $user->getId() && !$user->hasRole('admin'))
        $app->halt(403, 'You are not allowed to view this match');

    return $match;
}

function render_match_action($action) {
    return function($id) use ($action) {
        global $app;
        $app->render("match/$action", array('match' => find_match($id)));
    };
}

function map_format($format, $array) {
    $arr = array();

    foreach ($array as $key => $value)
        $arr[$key] = sprintf($format, $value);

    return $arr;
}

function filter_form($matches) {
    global $db;

    $form = new Form;
    $form->setRenderer(new BootstrapRenderer);
    $form->setAction('matches#results');
    $form->setMethod('get');

    $match_ids = array_unique($matches->fetchPairs(null, 'id'));
    $tags = $db->table('tag')
        ->select('DISTINCT name AS name')
        ->where(array('match_id' => $match_ids))
        ->fetchPairs('name', 'name');
    asort($tags);

    $distances = $matches->fetchPairs('distance', 'distance');
    asort($distances);
    $distances = map_format('%d meters', $distances);

    $form->addSelect('distance', 'Distance', $distances)
        ->setPrompt('Select a distance');
    $form->addSelect('tag', 'Tag', $tags)
        ->setPrompt('Select a tag');
    $form->addText('name', 'Name');
    $form->addSubmit('send', 'Filter');

    return $form;
}

function create_match_form() {
    global $db, $user;

    $disciplines = array('barebow', 'recurve', 'compound');
    $disciplines = array_combine($disciplines, $disciplines);

    $form = new Form;
    $form->setRenderer(new BootstrapRenderer);
    $form->setAction('match');

    $form->addText('name', 'Name')
        ->setRequired();
    $form->addText('created_at', 'Date')
        ->setType('datetime-local')
        ->setDefaultValue(strftime('%Y-%m-%dT%H:%M'))
        ->setAttribute('placeholder', 'YYYY-MM-DD HH:MM')
        ->addRule(Form::PATTERN, 'Invalid date', '\d{4}-\d{1,2}-\d{1,2}[T ]\d{1,2}:\d{1,2}')
        ->setRequired();
    $form->addText('distance', 'Distance')
        ->setAttribute('placeholder', 'distance in meters')
        ->setType('number')
        ->setRequired();
    $form->addRadioList('discipline', 'Discipline', $disciplines)
        ->setAttribute('data-inline', true)
        ->setRequired();
    $form->addText('arrows', 'Arrows')
        ->setType('range')
        ->setAttribute('min', 1)
        ->setAttribute('max', 10)
        ->setDefaultValue(3)
        ->setRequired();
    $form->addText('turns', 'Turns')
        ->setType('number')
        ->setAttribute('min', 1)
        ->setDefaultValue(10)
        ->setRequired();
    $form->addTextarea('notes', 'Notes');
    $form->addSubmit('send', 'Create');

    // Set attributes from last match as default, since the shooter is likely
    // to repeat the previous training in many cases
    $last_match = $db->table('match')
        ->where(array('user_id' => $user->getId()))
        ->order('created_at DESC')
        ->fetch();

    if ($last_match) {
        //$form['name']->setDefaultValue($last_match->name);
        $form['distance']->setDefaultValue($last_match->distance);
        $form['discipline']->setDefaultValue($last_match->discipline);
        $form['arrows']->setDefaultValue($last_match->arrows);
        $form['turns']->setDefaultValue($last_match->turns);
    }

    return $form;
}


$app->get('/matches', function ($filter=null) use ($app, $db, $user) {
    $matches = $db->table('match')
        ->where(array('user_id' => $user->getId()))
        ->order('created_at DESC');

    $filter_form = filter_form($matches);

    if ($filter_form->isSubmitted()) {
        $values = $filter_form->getValues();

        if ($values->tag) {
            $ids = $db->table('tag')->select('match_id')->where('name', $values->tag);
            $matches->where('id', $ids);
        }

        if ($values->distance)
            $matches->where('distance', $values->distance);

        if ($values->name)
            $matches->where('name LIKE ?', "%$values->name%");

        //foreach ($filter_form->getValues() as $prop => $value) {
        //$matches = $matches->where($filter_form->getValues();
    }

    $app->render('match/list', compact('matches', 'filter_form'));
});
$app->get('/match/:id', render_match_action('view'));
$app->get('/match/:id/edit', render_match_action('edit'));
$app->get('/match/:id/scores', render_match_action('scores'));
$app->get('/match/:id/delete', render_match_action('delete'));

$app->get('/match', function () use ($app) {
    $app->render('match/create', array('form' => create_match_form()));
});
$app->post('/match', function () use ($app, $db, $user) {
    $form = create_match_form();
    $form->validate();

    if (!$form->hasErrors()) {
        $values = $form->getValues();
        $values->created_at = str_replace('T', ' ', $values->created_at) . ':00';
        $values->scores = pack_scores(array_fill(0, $values->turns * $values->arrows, 0));
        $values->user_id = $user->getId();

        $match = $db->table('match')->insert($values);
        $app->redirect(ROOT_URL . "/match/{$match->id}/scores");
    }

    $app->render('match/create', compact('form'));
});

$app->post('/match/:id/scores', function ($id) use ($app, $db) {
    find_match($id)->update(array(
        'scores' => pack_scores(array_map('intval', $_POST['scores'])),
        'notes' => $_POST['notes']
    ));
    $app->redirect(ROOT_URL . '/match/' . $id);
});

$app->delete('/match/:id', function ($id) {
    find_match($id)->delete();
    $app->redirect(ROOT_URL . '/matches');
});

$app->put('/match/:id', function ($id) {
    // TODO
});
